<?php

namespace Controllers;

use Services\ConnectionIntegraWebService;
use Services\ReportOrderService as Service;
use Services\MakeErrorReport as ErrorReport;
use Controllers\SendEmailController;
use Util\XMLUtil;

class ReportOrderController
{

    public function __construct(Service $service, ConnectionIntegraWebService $webService, ErrorReport $errorReport, SendEmailController $sendEmail, XMLUtil $xmlUtil)
    {
        $this->service = $service;
        $this->webService = $webService;
        $this->errorReport = $errorReport;
        $this->sendEMail = $sendEmail;
        $this->xmlUtil = $xmlUtil;
    }

    public function reportOrderSuccess($nrAutoForn, $cdSupplier)
    {
        $numOrder = self::checkNumOrder($nrAutoForn);

        if (!empty($numOrder)) {
            $xml = self::generatesReportSuccess($numOrder, $nrAutoForn, $cdSupplier);
            self::sendToWebService($xml, $cdSupplier);
        }
    }

    public function reportOrderError($numOrder, $cdSupplier)
    {
        $errorLog = $this->service->getLogError($numOrder);
        $xml = self::generatesReportError($numOrder, $cdSupplier, $errorLog);
        self::sendToWebService($xml, $cdSupplier);
    }

    public function generatesXMLReport($cdSupplier, $arrayToConvert)
    {
        $xml = $this->xmlUtil->createXMLElement();
        $xml = self::setDataSupplierOnXML($cdSupplier, $xml);
        $xml = $this->xmlUtil->addObjectOnXML($arrayToConvert, $xml);
        $xml_file = $xml->asXML();
        return $xml_file;
    }

    public function generatesReportSuccess($numOrder, $nrAutoForn, $cdSupplier)
    {
        $arrayToConvert = [
            'pedido' => $numOrder,
            'ordem-compra' => $nrAutoForn,
        ];
        return self::generatesXMLReport($cdSupplier, $arrayToConvert);
    }

    public function generatesReportError($numOrder, $cdSupplier, $errorLog)
    {
        $strError = "";
        $strError = self::convertDataSetError($errorLog, $strError);
        $arrayToConvert = [
            'pedido' => $numOrder,
            'erro' => $strError,
        ];
        return self::generatesXMLReport($cdSupplier, $arrayToConvert);
    }

    public function convertDataSetError($errors, $strError)
    {
        foreach ($errors as $error) {
            $strError = $strError . $error['DSERROIMPORTAF'].'; ';
        }
        return $strError;
    }

    public function setDataSupplierOnXML($cdSupplier, $xml)
    {
        $paramIntegra = $this->service->getParamsIntegra($cdSupplier);
        $nmSupplier = $this->service->getSupplier($cdSupplier);
        $xml->addChild('nome-remetente', 'SAPORE');
        $xml->addChild('nome-destinatario', $nmSupplier);
        $xml->addChild('key', $paramIntegra['DSLOGIN']);
        $xml->addChild('auth', $paramIntegra['DSSENHA']);
        return $xml;
    }

    public function checkNumOrder($nrAutoForn)
    {
        $numOrder = $this->service->getNumOrder($nrAutoForn);
        return isset($numOrder[0]['USR_NRIMPORTAF']) ? $numOrder[0]['USR_NRIMPORTAF'] : "";
    }

    public function sendToWebService($xml, $cdSupplier)
    {
        $service = $this->webService->getAvailableServices($cdSupplier);
        $this->webService->connectWebService($service[0], (string) $xml);
    }

    public function sendReportMail($cdSupplier, $orders)
    {   
        $mailParams = [];
        foreach($orders as $order) {
            $report = $this->errorReport->makeReportFile($order);
            array_push($mailParams, [
                'numOrder' => $order['numOrder'],
                'report' => $report
            ]);
        }
        $cdoperador = $this->service->getUserLog($cdSupplier);
        if(isset($cdoperador[0])) {  
            $params = $this->service->getUserLogParams($cdoperador[0]['CDOPERADOR']);
            if(isset($params[0])) {
                $supplierName = $this->service->getSupplier($cdSupplier);
                $this->sendEMail->sendErrorReport($params[0], $cdSupplier, $supplierName, $mailParams);
            }
        }
    }

}
