<?php
namespace Modules\Fornecimento\Modules\MigracaoAF\Controllers;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Modules\Fornecimento\Modules\MigracaoAF\Services\MigracaoAFService;

class MigracaoAFController extends \Teknisa\GER\Controllers\TeknisaCrud
{

    protected $crudService;
    protected $dataSourceManager;
    protected $entityManager;

    public function __construct($crudService, $parameterBag, $dataSourceManager, EntityManager $entityManager, MigracaoAFService $service, $supUtilities)
    {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->entityManager = $entityManager;
        $this->service = $service;
        $this->supUtilities = $supUtilities;
	}

    public function getCentralCompras(DTO\Request\Row $request, DTO\Response $response)
    {
        $resposta = $this->crudService->getFilteredDataSet("central_compras_default", $this->service->criaFiltroCentralCompras())->getRows();
        $response->addDataSet(new DataSet("resposta", $resposta));
    }

    public function getFornecedores(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $resposta  = $this->getMigracaoAF($row);
        $response->addDataSet(new DataSet("resposta", $resposta));
    }

    public function getMigracaoAF($row)
    {
        return $this->crudService->getFilteredDataSet("sup98000_migracaoAF", $this->service->criaFiltroMigracaoAF($row))->getRows();
    }

    public function getProdutoAF(DTO\Request\Filter $request, DTO\Response $response)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
        $resposta = $this->crudService->getFilteredDataSet("sup98000_produtoAF", $conditions)->getRows();
        $response->addDataSet(new DataSet("resposta", $resposta));
    }

    public function getPesqFornAtivo(DTO\Request\Filter $request, DTO\Response $response)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
        $resposta = $this->crudService->getFilteredDataSet("sup98000_pesqFornAtivo", $conditions)->getRows();
        $response->addDataSet(new DataSet("resposta", $resposta));
    }

    protected function getItemCole($row)
    {
        return $this->crudService->getFilteredDataSet("sup98000_confirmaMigracao", $this->service->criaFiltroItemCole($row))->getRows();
    }

    public function getConfirmaMigracao(DTO\Request\DataSet $request, DTO\Response $response)
    {
        $row = $request->getDataSet()->getRows();
        $validacao = $row[0][0];
        $rows = $row[1];

        $arrayItemCole = array();

        if ($validacao) {
            $prodAF = $this->crudService->getFilteredDataSet("sup98000_produtoAF", $this->service->filtroProduto($rows))->getRows();
        } else {
            $prodAF = $rows;
        }
        foreach ($prodAF as $row) {
            $comprador = $this->service->getComprador($row)[0];
            $row['CDFORNECED']   = $rows[0]['NEW_CDFORNECED'];
            $row['IDGERAFDTENT'] = $rows[0]['NEW_IDGERAFDTENT'];
            $row['IDTPIJURFORN'] = $rows[0]['NEW_IDTPIJURFORN'];
            $row['NMFANTFORN']   = $rows[0]['NEW_NMFANTFORN'];
            $row['NMRAZSOCFORN'] = $rows[0]['NEW_NMRAZSOCFORN'];
            $row['NRINSJURFORN'] = $rows[0]['NEW_NRINSJURFORN'];
            $row['IDTIPOEMITAF'] = $rows[0]['IDTIPOEMITAF'];
            $row['CDEMPRESA']    = $rows[0]['CDEMPRESA'];
            $row['IDTIPOLANCAF'] = $rows[0]['IDTIPOLANCAF'];
            $row['CDMARCA_OLD']     = $row['CDMARCA'];
            $row['CDPRODUTO_OLD']   = $row['CDPRODUTO'];
            $row['DTPROGENTAF_OLD'] = $row['DTPROGENTAF'];
            $row['CDOPERADOR']      = $comprador['CDOPERADOR'];
            $resposta = $this->getItemCole($row);
            if(sizeof($resposta) > 0){
                $row['VRPREUNIAF']  = $resposta[0]['VRPRECUNIITM'];
                $row['CDMARCA']     = $resposta[0]['CDMARCA'];
                $row['NMMARCA']     = $resposta[0]['NMMARCA'];
            } else {
                $row['VRPREUNIAF']  = 0;
            }
            array_push($arrayItemCole, $row);
        }
        $response->addDataSet(new DataSet("resposta", $arrayItemCole));
    }

    public function getPesquisaProduto(DTO\Request\Filter $request, DTO\Response $response)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
        $resposta = $this->crudService->getFilteredDataSet("sup98000_pesquisaProduto", $conditions)->getRows();
        $response->addDataSet(new DataSet("resposta", $resposta));
    }

    public function getPesqMarcaProduto(DTO\Request\Filter $request, DTO\Response $response)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
        $resposta = $this->crudService->getFilteredDataSet("sup98000_pesqMarcaProduto", $conditions)->getRows();
        $response->addDataSet(new DataSet("resposta", $resposta));
    }

    public function getNovoPreco(DTO\Request\DataSet $request, DTO\Response $response)
    {
        $rows = $request->getDataSet()->getRows();
        $resposta = $this->getItemCole($rows[0]);
        $response->addDataSet(new DataSet("resposta", $resposta));
    }

    protected function validaMigracaoAF($rows, $messages)
    {
        $arrayError = array();
        $error = null;
        foreach ($rows as $row) {
            $row['CDFILCENTCMP'] = $this->supUtilities->getCentralCompras($row['CDFILDEST'])['CDFILCENTCMP'];

            //Valida Data de Entrega
            $data1 = date_format(date_create_from_format( "d/m/Y", $row['DTPROGENTAF']), "Y-m-d");
            $data2 = date('Y-m-d');
            if($data1 < $data2){
                $error = $this->service->preencheDsError($row, 'DT_Prod_Inv', $messages);
                $arrayError = $this->service->gravaError($error, $arrayError);
                $error['DSERROR'] = null;
            }

            //Valida produto
    		$fornOL = $this->crudService->getFilteredDataSet("sup98000_fornecedorOL", $this->service->filtroFornOLProduto($row))->getRows();
            if(sizeOf($fornOL) > 0){
                $row['CDFORNOPELOG'] = $fornOL[0]['CDFORNOPELOG'];
            } else {
                $row['CDFORNOPELOG'] = null;
            }
            $produto = $this->crudService->getFilteredDataSet("sup98000_pesquisaProduto", $this->service->filtroPesqProduto($row))->getRows();
            $error = array();
            if(sizeOf($produto) == 0){
                if($row['CDFORNOPELOG']  == null){
                    if(isset($row['IDUTLPROCMPAF']) && $row['IDUTLPROCMPAF'] == 'N'){
                        $error = $this->service->preencheDsError($row, 'ProdNCad', $messages);
                    } else {
                        $error = $this->service->preencheDsError($row, 'Prod_est', $messages);
                    }
                } else {
                    $error = $this->service->preencheDsError($row, 'ProdEstoInval', $messages);
                }
            }
            if(!empty($error[0])) {
                if($error['DSERROR'] != null) {
                    $arrayError = $this->service->gravaError($error, $arrayError);
                    $error['DSERROR'] = null;
                }
            }

            //Valida Marca
            $marca = $this->crudService->getFilteredDataSet("sup98000_pesqMarcaProduto", $this->service->filtroMarcaProduto($row))->getRows();
            if(sizeOf($marca) == 0){
                $error = $this->service->preencheDsError($row, 'PesqMarcaProd', $messages);
                $arrayError = $this->service->gravaError($error, $arrayError);
                $error['DSERROR'] = null;
            }

            /*Produto_Inativo|Produto(s) informado(s) inativo(s)*/
            $resposta = $this->crudService->getFilteredDataSet("sup98000_validaProdAtivo", $this->service->criaFiltroProdAtivo($row))->getRows();
            if(sizeOf($resposta) == 0){
                $error = $this->service->preencheDsError($row, 'Produto_Inativo', $messages);
                $arrayError = $this->service->gravaError($error, $arrayError);
                $error['DSERROR'] = null;
            }

            /*Marc_NQualif|Marca não qualificada ou com validade vencida.*/
            $resposta = $this->crudService->getFilteredDataSet("sup98000_validaQualifMarca", $this->service->criaFiltroQualifMarca($row))->getRows();
            if(sizeOf($resposta) > 0){
                $error = $this->service->preencheDsError($row, 'Marc_NQualif', $messages);
                $arrayError = $this->service->gravaError($error, $arrayError);
                $error['DSERROR'] = null;
            }

            /*N_MarcProd|Marca(s) informada(s) não cadastrada(s) ou não qualificada(s) para o(s) produto(s) informado(s).*/
            $resposta = $this->crudService->getFilteredDataSet("sup98000_validaQualifMarcProd", $this->service->criaFiltroQualifMarcProd($row))->getRows();
            if(sizeOf($resposta) > 0){
                $error = $this->service->preencheDsError($row, 'N_MarcProd', $messages);
                $arrayError = $this->service->gravaError($error, $arrayError);
                $error['DSERROR'] = null;
            }

            /*N_ProdFornComp| Produto informado nao cadastrado para o fornecedor*/
            $resposta = $this->crudService->getFilteredDataSet("sup98000_validaProdForneComp", $this->service->criaFiltroProdForneComp($row))->getRows();
            if(sizeOf($resposta) == 0){
                $error = $this->service->preencheDsError($row, 'N_ProdFornComp', $messages);
                $arrayError = $this->service->gravaError($error, $arrayError);
                $error['DSERROR'] = null;
            }

            /*N_RefoReg|Fornecedor não atende à(s) região(ões)/filial(is) informada(s).*/
            $resposta = $this->crudService->getFilteredDataSet("sup98000_validaReFoReg", $this->service->criaFiltroReFoReg($row))->getRows();

            if(sizeOf($resposta) > 0){
                $error = $this->service->preencheDsError($row, 'N_RefoReg', $messages);
                $arrayError = $this->service->gravaError($error, $arrayError);
                $error['DSERROR'] = null;
            } else {
                $resposta = $this->crudService->getFilteredDataSet("sup98000_validaReForfil", $this->service->criaFiltroReForfil($row))->getRows();
                if(sizeOf($resposta) > 0){
                    $error = $this->service->preencheDsError($row, 'N_RefoReg', $messages);
                    $arrayError = $this->service->gravaError($error, $arrayError);
                    $error['DSERROR'] = null;
                }
            }

            /*Marca Obrigatoria / Marca Preferencial / Restricao a Marca*/
            $fornecedorOL = $this->crudService->getFilteredDataSet("sup98000_validaFornecedorOL", $this->service->criaFiltroFornecedorOL($row))->getRows();
            if(sizeOf($fornecedorOL) > 0){
                $particMarca = $this->crudService->getFilteredDataSet("sup98000_validaParticMarca", $this->service->criaFiltroParticMarca($row))->getRows();
                if(sizeOf($particMarca) > 0){
                    $marcaPref = null;
                    foreach ($particMarca as $partic){
                        /*Marca Obrigatoria*/
                        if($partic['CDTIPPART'] == '1'){
                            if($partic['CDMARCA'] != $row['CDMARCA']){
                                $error = $this->service->preencheDsError($row, 'MarcaObrig', $messages);
                                $arrayError = $this->service->gravaError($error, $arrayError);
                                $error['DSERROR'] = null;
                                break;
                            }
                        } else {
                            /*Marca Preferencial*/
                            if($partic['CDTIPPART'] == '2'){
                                if($partic['CDMARCA'] == $row['CDMARCA']){
                                    $marcaPref = '0';
                                } else {
                                    if($marcaPref != '0'){
                                        $marcaPref = '1';
                                    }
                                }
                            } else {
                                /*Restricao a Marca*/
                                if($partic['CDTIPPART'] == '3'){
                                    if($partic['CDMARCA'] == $row['CDMARCA']){
                                        $error = $this->service->preencheDsError($row, 'RestricMarca', $messages);
                                        $arrayError = $this->service->gravaError($error, $arrayError);
                                        $error['DSERROR'] = null;
                                        break;
                                    }
                                }
                            }
                        }
                    }
                    if($marcaPref != null){
                        $error = $this->service->preencheDsError($row, 'MarcaPref', $messages);
                        $arrayError = $this->service->gravaError($error, $arrayError);
                        $error['DSERROR'] = null;
                    }
                }
            }

            /*Produto Preferencial*/
            /*$resposta = $this->crudService->getFilteredDataSet("sup98000_validaProdPref", $this->service->criaFiltroProdPref($row))->getRows();
            if(sizeOf($resposta) > 0){
                if($resposta[0]['CDPRODPREF'] != $row['CDPRODUTO']){
                    $error = $this->service->preencheDsError($row, 'ProdPref', $messages);
                    $arrayError = $this->service->gravaError($error, $arrayError);
                    $error['DSERROR'] = null;
                }
            }*/

            /*Periodicidade Fornecedor*/
            /*$resposta = $this->service->procedureCarrega_PeriForn($row);
            if(sizeOf($resposta) > 0){
                if($resposta['P_NEW_DTPROGENTAF'] != $row['DTPROGENTAF']){
                    $error = $this->service->preencheDsError($row, 'PeriForn', $messages);
                    $arrayError = $this->service->gravaError($error, $arrayError);
                    $error['DSERROR'] = null;
                }
            }*/

            /*Produto não cadastrado para o comprador*/

            $resposta = $this->crudService->getFilteredDataSet("sup98000_validaVer_ProdCompf", $this->service->criaFiltroVer_ProdComp($row))->getRows();
            if(sizeOf($resposta) > 0){
                $resposta1 = intval($resposta[0]['CONTA']);
                if($resposta1 === 0){
                    $error = $this->service->preencheDsError($row, 'ProdComprador', $messages);
                    $arrayError = $this->service->gravaError($error, $arrayError);
                    $error['DSERROR'] = null;
                }
            }

        }
        return $arrayError;
    }

    public function setMigracaoAF(DTO\Request\DataSet $request, DTO\Response $response)
    {
        $rows = $request->getDataSet()->getRows();
        $arrayError = $this->validaMigracaoAF($rows[1], $rows[0]);

        if(sizeOf($arrayError) == 0){
            $nrautoforn = '          ';
            $stChaveAF = null;
            $stChaveItemAF = null;
            $arrayAF = array();
            foreach ($rows[1] as $row) {
                $cdFilCentCMP = $this->supUtilities->getCentralCompras($row['CDFILDEST']);
                $row['CDFILCENTCMP'] = $cdFilCentCMP['CDFILCENTCMP'];
                if($row['IDGERAFDTENT'] == 'S'){
                    $stData = $row['DTPROGENTAF'];
                } else {
                    $stData = null;
                }
                //Insere AF
                if($stChaveAF != $row['CDFILDEST'] . $stData){
                    $stChaveAF = $row['CDFILDEST'] . $stData;
                    $fornecedorOL = $this->crudService->getFilteredDataSet("sup98000_validaFornecedorOL", $this->service->criaFiltroFornecedorOL($row))->getRows();
                    $nrautoforn = $this->service->salvarAF($row, $rows[2], $fornecedorOL);

                    if($rows[2]['IDTPFORPGTFO'] == 'V'){
                        $this->service->salvarPrazoPgtAF($row, $nrautoforn);
                    }
                    $af['CDFILDEST']     = $row['CDFILDEST'];
                    $af['CDFILLANC']     = $cdFilCentCMP['CDFILCENTCMP'];
                    $af['NRAUTOFORN']    = $nrautoforn;
                    $af['CDFILLANCORIG'] = $row['CDFILLANC'];
                    $af['NRAUTOFORNORI'] = $row['NRAUTOFORN'];
                    array_push($arrayAF, $af);
                }
                
                //Insere Item de AF
                if($stChaveItemAF != $stChaveAF . $row['CDPRODUTO'] . $row['CDMARCA'] . $row['DTPROGENTAF']){
                    $stChaveItemAF = $stChaveAF . $row['CDPRODUTO'] . $row['CDMARCA'] . $row['DTPROGENTAF'];
                    $itemAFNova = $this->crudService->getFilteredDataSet("sup98000_itemAFNova", $this->service->criaFiltroItemAFNova($row, $nrautoforn))->getRows();
                    $this->service->salvarItemAFMigracao($row, $nrautoforn, $itemAFNova);
                    $dbQtdOld = $row['QTPROGENTAF'];
                } else {
                    $dbQtdOld = $dbQtdOld + $row['QTPROGENTAF'];
                    $this->service->updateItemAFMigra($row, $nrautoforn, $dbQtdOld);
                }
                $this->service->updateItemAFOrigem($row);
            }
            $stChaveNrAF = null;
            $stChaveNrAFOrig = null;

            foreach ($arrayAF as $af) {
                if($stChaveNrAF != $af['NRAUTOFORN']){
                    $stChaveNrAF = $af['NRAUTOFORN'];
                    $verifCtPag = $this->service->adqVerifCtPag($af)[0];
                    if($verifCtPag <= 1) {
                        $this->service->updateTpCtPagar($af);
                    }else{
                        $this->service->updateTpCtPagar1($af);
                    }
                    $this->service->procedureFechamentoAf($af['CDFILDEST'], $af['CDFILLANC'], $af['NRAUTOFORN']);
                    //Gera conta a pagar da AF nova
                    $excluiContaPrev = $this->service->procedureExcluiContaPagarPrevista($af['CDFILDEST'], $af['CDFILLANC'], $af['NRAUTOFORN']);
                    $this->service->procedureGeraContaPagarPrevista($af['CDFILDEST'], $af['CDFILLANC'], $af['NRAUTOFORN']);

                }
                if($stChaveNrAFOrig != $af['NRAUTOFORNORI']){
                    $stChaveNrAFOrig = $af['NRAUTOFORNORI'];
                    $itemBaixado = $this->crudService->getFilteredDataSet("sup98000_itemAFOriginalBaixa", $this->service->filtroAF($af))->getRows();
                    if($itemBaixado > 0){
                        $idBaixaAF = '1';
                    } else {
                        $idBaixaAF = '3';
                    }

                    $this->service->updateBaixaAf($af, $idBaixaAF);
                    //AJUSTA CONTA A PAGAR DA AF ANTIGA
                    $this->service->procedureFechamentoAf($af['CDFILDEST'], $af['CDFILLANCORIG'], $af['NRAUTOFORNORI']);
                    $excluiContaPrev = $this->service->procedureExcluiContaPagarPrevista($af['CDFILDEST'], $af['CDFILLANCORIG'], $af['NRAUTOFORNORI']);
                    $this->service->procedureGeraContaPagarPrevista($af['CDFILDEST'], $af['CDFILLANCORIG'], $af['NRAUTOFORNORI']);
                }
            }
            $array = array();
            array_push($array, /*$erro . */$nrautoforn);
            $response->addDataSet(new DataSet("resposta", $array));
        } else {
            $response->addDataSet(new DataSet("resposta", $arrayError));
        }

    }
}